function net = cnn_VDSR_init(varargin)
% Initialize a CNN  for VDSR
% Different to SRCNN each conv layer using weights: 64x3x3x64

opts.batchNormalization = true ;
opts.networkType = 'simplenn' ;
opts = vl_argparse(opts, varargin) ;

rng('default');
rng(0) ;

totalL=20;% <------------parameter here

f=1/100 ;
net.layers = {} ;

% Input layer
net.layers{end+1} = struct('type', 'conv', ...
                           'weights', {{f*randn(3,3,1,64, 'single'), zeros(1, 64, 'single')}}, ...
                           'stride', 1, ...
                           'pad', 1) ;
net.layers{end+1} = struct('type', 'relu') ; 

% The same convolutional layer
for L=1:totalL-2
    net.layers{end+1} = struct('type', 'conv', ...
                           'weights', {{f*randn(3,3,64,64, 'single'),  zeros(1,64,'single')}}, ...
                           'stride', 1, ...
                           'pad', 1) ;
    net.layers{end+1} = struct('type', 'relu') ;   
end

% Output layer and Loss function (L2-norm)
net.layers{end+1} = struct('type', 'conv', ...
                           'weights', {{f*randn(3,3,64,1, 'single'),  zeros(1,1,'single')}}, ...
                           'stride', 1, ...
                           'pad', 1) ;
          
net.layers{end+1} = struct('type', 'pdist', ...
                            'p',2, ...
                            'noRoot', true) ; % Using l2-norm as Loss function

% optionally switch to batch normalization
if opts.batchNormalization
  net = insertBnorm(net, 1) ;
  
  ii=1;
  for iL=1:totalL-1
      ii=ii+3;
      net = insertBnorm(net, ii) ;
  end

end

% Meta parameters
net.meta.inputSize = [36 36 1] ;
net.meta.trainOpts.learningRate = 0.00001 ;
net.meta.trainOpts.numEpochs = 20;
net.meta.trainOpts.batchSize = 64 ;

% Fill in defaul values
net = vl_simplenn_tidy(net) ;

% Switch to DagNN if requested
switch lower(opts.networkType)
  case 'simplenn'
    % done
  case 'dagnn'
%     net = dagnn.DagNN.fromSimpleNN(net, 'canonicalNames', true) ;
%     net.addLayer('top1err', dagnn.Loss('loss', 'classerror'), ...
%       {'prediction', 'label'}, 'error') ;
%     net.addLayer('top5err', dagnn.Loss('loss', 'topkerror', ...
%       'opts', {'topk', 5}), {'prediction', 'label'}, 'top5err') ;
    assert(false) ;
  otherwise
    assert(false) ;
end

% --------------------------------------------------------------------
function net = insertBnorm(net, l)
% --------------------------------------------------------------------
assert(isfield(net.layers{l}, 'weights'));
ndim = size(net.layers{l}.weights{1}, 4);
layer = struct('type', 'bnorm', ...
               'weights', {{ones(ndim, 1, 'single'), zeros(ndim, 1, 'single')}}, ...
               'learningRate', [1 1 0.05], ...
               'weightDecay', [0 0]) ;
net.layers{l}.biases = [] ;
net.layers = horzcat(net.layers(1:l), layer, net.layers(l+1:end)) ;
